(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'query-string', './utils'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('query-string'), require('./utils'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.queryString, global.utils);
    global.apiClient = mod.exports;
  }
})(this, function (exports, _queryString, _utils) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.ApiClient = exports.sharedHeaders = undefined;

  var qs = _interopRequireWildcard(_queryString);

  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};

      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
        }
      }

      newObj.default = obj;
      return newObj;
    }
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  var sharedHeaders = exports.sharedHeaders = {
    'Content-Type': 'application/json',
    'Accept': 'application/json',
    'X-Requested-With': 'XMLHttpRequest'
  };

  // Helpers
  // JSON parser
  // const parseJson = (resp) => resp.json();
  var parseJson = function parseJson(state, opts) {
    return function (resp) {
      return resp.json();
    };
  };

  // MIDDLEWARE THINGS
  var checkStatus = function checkStatus(resp) {
    if (!resp.ok) {
      var err = new Error(resp.statusText);
      err.status = resp.status;
      err.response = resp;
      err.body = Promise.resolve((0, _utils.noop)(resp)).then(function (resp) {
        return resp.json();
      }).catch(function (err) {
        return Promise.reject({ msg: 'Service error' });
      });
      throw err;
    }
    return resp;
  };

  /**
   * API Client class
   *
   * baseOpts
   *  {
   *    baseUrl: Required,
   *    appendExt: boolean || string
   *  }
   */

  var ApiClient = exports.ApiClient = function () {
    function ApiClient(baseOpts, getState) {
      var _this = this;

      _classCallCheck(this, ApiClient);

      // basically makeRequest
      this._debugging = baseOpts._debug;

      ['GET', 'POST', 'PUT', 'PATCH', 'DELETE', 'HEAD'].forEach(function (method) {
        _this[method.toLowerCase()] = function (options) {
          var opts = _this.requestDefaults(method, options, baseOpts);

          var url = _this._getUrl(options, baseOpts);

          var requestTransforms = _this._parseOpt('requestTransforms', options, baseOpts, []);
          var responseTransforms = _this._parseOpt('responseTransforms', options, baseOpts, [parseJson]);

          // let requestTransforms = [].concat(reqTransforms);
          // let responseTransforms = [].concat(respTransforms);

          return new Promise(function (resolve, reject) {
            _this.runTransforms(requestTransforms, getState, opts).then(_this.debugLog(function () {
              return 'requesting...';
            })).then(function (transformedOpts) {
              return fetch(url, transformedOpts);
            }).then(_this.debugLog()).then(checkStatus).then(_this.debugLog(function (resp) {
              return 'Status: ' + resp;
            })).then(function (resp) {
              return _this.runTransforms(responseTransforms, getState, resp);
            }).then(_this.debugLog(function (json) {
              return 'JSON: ' + json;
            })).then(resolve).catch(function (err) {
              return reject(err);
            });
          });
        };
      });
    }

    _createClass(ApiClient, [{
      key: 'runTransforms',
      value: function runTransforms(transforms, getState, opts) {
        return (0, _utils.syncEach)((0, _utils.noop)(opts), function (fn, prev) {
          return typeof fn === 'function' ? fn(getState, opts) : fn;
        })(transforms);
      }
    }, {
      key: 'debugLog',
      value: function debugLog(msgFn) {
        var _this2 = this;

        return function (args) {
          if (_this2._debugging) {
            console.log((msgFn || _utils.noop)(args));
          }
          return args;
        };
      }

      // Option handling

    }, {
      key: '_parseOpt',
      value: function _parseOpt(key, opts) {
        var defaultOpts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
        var defaultValue = arguments.length <= 3 || arguments[3] === undefined ? null : arguments[3];

        var val = opts[key] || defaultOpts[key];
        if (!val) {
          return defaultValue;
        }

        return typeof val === 'function' ? val.call(this, opts) : val;
      }
    }, {
      key: 'requestDefaults',
      value: function requestDefaults(method, params, defaultOpts) {
        var opts = params || {};

        var customHeaders = this._parseOpt('headers', opts, defaultOpts, {});
        var headers = Object.assign({}, sharedHeaders, customHeaders);

        var meta = this._parseOpt('meta', opts, defaultOpts, {});
        var data = this._parseOpt('data', opts, defaultOpts, {});

        var requestOpts = {
          method: method,
          headers: headers,
          meta: meta
        };

        if (method === 'POST' || method === 'PUT' || method === 'PATCH') {
          requestOpts.body = JSON.stringify(data);
        }

        return requestOpts;
      }

      // Get custom url if included in the opts
      // otherwise, get the default one

    }, {
      key: '_getUrl',
      value: function _getUrl(opts) {
        var defaultOpts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

        if (typeof opts === 'string') {
          opts = { path: opts };
        }
        var url = this._parseOpt('url', opts);
        var path = this._parseOpt('path', opts);

        if (path) {
          path = path[0] !== '/' ? '/' + path : path;
        }

        if (!url) {
          url = [defaultOpts.baseUrl, path].join(''); // eslint-disable-line no-undef
        }

        var appendPath = this._parseOpt('appendPath', opts, defaultOpts);
        if (appendPath) {
          url = [url, appendPath].join('');
        }

        // Append (or don't)
        var appendExt = this._parseOpt('appendExt', opts, defaultOpts);
        if (appendExt) {
          url = url + ('.' + appendExt);
        }

        var params = this._parseOpt('params', opts, defaultOpts);
        if (params) {
          var qsParams = qs.stringify(params);
          url = url + '?' + qsParams;
        }

        return url;
      }
    }]);

    return ApiClient;
  }();

  exports.default = ApiClient;
});