(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'invariant', './utils'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('invariant'), require('./utils'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.invariant, global.utils);
    global.constants = mod.exports;
  }
})(this, function (exports, _invariant, _utils) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.REDUX_MODULE_API_ACTION_KEY = exports.REDUX_MODULE_ACTION_KEY = undefined;
  exports.apiValues = apiValues;
  exports.createCustomTypes = createCustomTypes;
  exports.createConstants = createConstants;

  var _invariant2 = _interopRequireDefault(_invariant);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj;
  };

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var REDUX_MODULE_ACTION_KEY = exports.REDUX_MODULE_ACTION_KEY = 'REDUX_MODULES/ACTION';
  var REDUX_MODULE_API_ACTION_KEY = exports.REDUX_MODULE_API_ACTION_KEY = 'REDUX_MODULES/API_ACTION';

  var customTypes = {
    api: _utils.apiStates
  };
  /*
   * get the api values
   */
  function apiValues(name) {
    var states = arguments.length <= 1 || arguments[1] === undefined ? _utils.apiStates : arguments[1];

    return (0, _utils.apiKeys)(name, states).reduce(function (sum, key) {
      return _extends({}, sum, _defineProperty({}, key, (0, _utils.toApiKey)(key)));
    }, {});
  }

  function createCustomTypes(name) {
    var states = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

    customTypes[name] = states;
  }

  function createConstants(opts) {
    opts = opts || {};
    var separator = opts.separator || '_';

    if (opts.customTypes) {
      Object.keys(opts.customTypes).forEach(function (key) {
        createCustomTypes(key, opts.customTypes[key]);
      });
    }

    if (typeof opts === 'string') {
      opts = { prefix: opts };
    }

    var defineType = function defineType(obj, prefix, n, v) {
      if ((typeof n === 'undefined' ? 'undefined' : _typeof(n)) === 'object') {
        // If it's an API type, decorate with the different
        // api states
        Object.keys(n).forEach(function (key) {
          if (n.hasOwnProperty(key)) {
            var val = n[key];
            if (val && val.types) {
              var types = Array.isArray(val.types) ? val.types : val.types.split(' ,');
              types.forEach(function (type) {
                // Define custom types for each
                var states = customTypes[type] || [];
                states.forEach(function (customKey) {
                  var name = (0, _utils.upcase)('' + key + separator + customKey);
                  var keyPrefix = (0, _utils.upcase)([type, customKey].join(separator));
                  var valuePrefix = [prefix, type].join(separator);
                  // const apiPrefix = `api_${prefix}`;
                  defineType(obj, valuePrefix, name);
                });
              });
            }
            if (val && val.api) defineApi(obj, prefix, key, val.states);

            // Don't store objects in static arrays
            if ((typeof val === 'undefined' ? 'undefined' : _typeof(val)) === 'object') {
              val = null;
            }

            defineType(obj, prefix, key, v);
          }
        });
      } else {
        v = v || [].concat.apply([], [prefix, n]).join(separator).toUpperCase();
        Object.defineProperty(obj, n, {
          value: v,
          enumerable: true,
          writable: false,
          configurable: false
        });
      }
      return obj;
    };

    var defineApi = function defineApi(obj, prefix, n, states) {
      (0, _utils.apiKeys)(n, states).forEach(function (apiKey) {
        var apiPrefix = 'api_' + prefix;
        defineType(obj, apiPrefix, apiKey);
      });
    };

    var definer = function definer(obj, prefix) {
      return function () {
        for (var _len = arguments.length, definitions = Array(_len), _key = 0; _key < _len; _key++) {
          definitions[_key] = arguments[_key];
        }

        definitions.forEach(function (def) {
          defineType(obj, prefix, def);
        });
        return obj;
      };
    };

    var initialObject = opts.initialObject || {};
    var prefix = opts.prefix || '';
    return definer(initialObject, prefix);
  }

  exports.default = createConstants;
});