(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'redux-actions', './constants', './apiClient', './utils'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('redux-actions'), require('./constants'), require('./apiClient'), require('./utils'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.reduxActions, global.constants, global.apiClient, global.utils);
    global.createApiActions = mod.exports;
  }
})(this, function (exports, _reduxActions, _constants, _apiClient, _utils) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.API_CONSTANTS = undefined;
  exports.createApiAction = createApiAction;
  exports.api = api;

  var _apiClient2 = _interopRequireDefault(_apiClient);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var API_CONSTANTS = exports.API_CONSTANTS = {
    401: 'API_ERROR_FOUR_OH_ONE',
    418: 'API_ERROR_I_AM_A_LITTLE_TEAPOT',
    // 422: 'API_ERROR_UNPROCESSABLE',
    UNKNOWN_ERROR: 'API_ERROR_UNKNOWN'
  };

  var getActionTypesForKeys = function getActionTypesForKeys(type) {
    var actionCreator = arguments.length <= 1 || arguments[1] === undefined ? _utils.noop : arguments[1];
    var metaCreator = arguments.length <= 2 || arguments[2] === undefined ? _utils.noop : arguments[2];
    return (0, _utils.getApiTypes)(type).reduce(function (memo, key, idx) {
      return _extends({}, memo, _defineProperty({}, _utils.apiStates[idx], (0, _reduxActions.createAction)((0, _utils.toApiKey)(key), actionCreator, function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return _extends({ isApi: true }, metaCreator(args));
      })));
    }, {});
  };

  // Define a decorator for a function defined in an object
  // that is expected to be an action.
  // @param type - string constant
  function createApiAction(type, requestTransforms, responseTransforms, metaCreator) {
    // The decorator function wrapper (design: decoratorFn(fn()))
    // @param target - function to be decorated
    // @param name - the function name
    // @param def - the Object.defineProperty definition
    //
    // Functionally, this works by accepting the object definition type
    // from ES5 syntax, i.e.
    //    Object.defineProperty(this, 'fetchAll', {configurable: false, value: 2})
    // and it manipulates the definition by changing the value to be a function
    // that wraps the different api states, aka LOADING, SUCCESS, ERROR
    return function decoration(target) {
      // ApiTypes is the string constants mapped to a
      // createdAction (using `createAction`)
      // i.e. { loading: fetchAllLoading(), // anonymouse function
      //        success: fetchAllSuccess(),
      //        error: fetchAllError(),
      //      }
      var apiTypes = getActionTypesForKeys(type, function (t) {
        return t;
      }, metaCreator);
      // The new value for the object definition
      // return function decorated(globalOpts) {
      // globalOpts = globalOpts || {};
      // return a function (for thunk)
      return function () {
        var configurationOpts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
        return function (dispatch, getState) {

          var autoExecute = true;
          if (typeof configurationOpts.autoExecute !== 'undefined') {
            autoExecute = configurationOpts.autoExecute;
            delete configurationOpts['autoExecute'];
          }

          // Small helper to turn all functions within this decorated functions
          // into a promise
          var promiseWrapper = function promiseWrapper(fn) {
            return function () {
              return Promise.resolve(dispatch(fn.apply(undefined, arguments)));
            };
          };

          // The default handlers for the different states of the api request
          var loading = promiseWrapper(apiTypes.loading);
          var success = promiseWrapper(apiTypes.success);
          var error = function error(errorObj) {
            var getErrorStatus = function getErrorStatus(status) {
              return typeof API_CONSTANTS[status] !== 'undefined' ? API_CONSTANTS[status] : API_CONSTANTS.UNKNOWN_ERROR;
            };

            var reduceError = function reduceError(err) {
              return dispatch(apiTypes.error({ error: errorObj, body: err }));
            };
            var errType = (0, _utils.toApiKey)((0, _utils.getApiTypes)(type)[2]);
            var errStatus = errorObj && errorObj.status ? getErrorStatus(errorObj.status) : API_CONSTANTS.UNKNOWN_ERROR;

            var action = {
              type: errType,
              payload: {
                error: errorObj,
                status: errStatus
              }
            };

            dispatch(action);
            return action;
          };

          var runFn = function runFn(runtimeOpts) {
            var opts = Object.assign({}, configurationOpts, runtimeOpts);
            // The `initializer` is the original pre-decorated function
            // let actionFn = def ? def.initializer(opts) : target;
            var actionFn = target;
            // Every action gets an instance of ApiClient
            var client = new _apiClient2.default(opts, getState, requestTransforms, responseTransforms);
            // NOTE, check this: do we need below version ?
            // new ApiClient(opts, getState, requestTransforms, responseTransforms);

            // callAction wraps the `async` functionality that ensures
            // we always get a promise returned (and can be used to pass along
            // other thunk functions)
            var callAction = function callAction() {
              loading(opts);
              var retVal = actionFn.call(null, client, opts, getState, dispatch);
              if (typeof retVal.then !== 'function') {
                retVal = Promise.resolve(retVal);
              }
              return retVal.then(success)
              // Should we _also_ fire the original event, even if it's an API request too?
              // .then((...args) => dispatch({type: type, payload: args}))
              .catch(error);
            };
            // NOTE, check this: do we need below version ?
            // new ApiClient(opts, getState, requestTransforms, responseTransforms);

            // callAction wraps the `async` functionality that ensures
            // we always get a promise returned (and can be used to pass along
            // other thunk functions)
            return callAction();
          };

          var action = {
            type: _constants.REDUX_MODULE_API_ACTION_KEY,
            meta: { runFn: runFn, type: type }
          };
          if (autoExecute) {
            dispatch(action);
          } else {
            return action;
          }
          // dispatch(action);
          // return action;
        };
      };
      // };
    };
  }

  // decorator form
  // @param type - string constant
  // @return function - a nwe decorated function Object.defineProperty
  function api(type, requestTransforms, responseTransforms, metaCreator) {
    // The decorator function wrapper (design: decoratorFn(fn()))
    // @param target - function to be decorated
    // @param name - the function name
    // @param def - the Object.defineProperty definition
    //
    // Functionally, this works by accepting the object definition type
    // from ES5 syntax, i.e.
    //    Object.defineProperty(this, 'fetchAll', {configurable: false, value: 2})
    // and it manipulates the definition by changing the value to be a function
    // that wraps the different api states, aka LOADING, SUCCESS< ERROR
    return function decoration(target, name, def) {
      var newVal = createApiAction(type, requestTransforms, responseTransforms, metaCreator)(def.initializer);
      var newDef = {
        enumerable: true,
        writable: true,
        configurable: true,
        value: newVal
      };

      return newDef;
    };
  }

  exports.default = api;
});