(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'invariant'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('invariant'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.invariant);
    global.utils = mod.exports;
  }
})(this, function (exports, _invariant) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.syncEach = exports.noop = exports.getApiTypes = exports.toApiKey = exports.upcase = exports.apiStates = undefined;
  exports.apiKeys = apiKeys;

  var _invariant2 = _interopRequireDefault(_invariant);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  /*
   * Default api states
   */
  var apiStates = exports.apiStates = ['loading', 'success', 'error'];

  var upcase = exports.upcase = function upcase(str) {
    return str.toUpperCase();
  };

  // Polyfill Array.isArray
  if (!Array.isArray) {
    Array.isArray = function (arg) {
      return Object.prototype.toString.call(arg) === '[object Array]';
    };
  }

  /*
   * apiKeys to fetch the api key type
   */
  function apiKeys(name) {
    var states = arguments.length <= 1 || arguments[1] === undefined ? apiStates : arguments[1];

    return states.map(function (state) {
      return upcase(name + '_' + state);
    });
  }

  var toApiKey = exports.toApiKey = function toApiKey(key) {
    return upcase('api_' + key);
  };
  var getApiTypes = exports.getApiTypes = function getApiTypes(type) {
    return apiKeys(type).reduce(function (memo, key) {
      return memo.concat(key);
    }, []);
  };

  var noop = exports.noop = function noop(r) {
    return r;
  };
  var syncEach = exports.syncEach = function syncEach(initial, cb) {
    (0, _invariant2.default)(initial, 'No initial value defined');

    cb = typeof cb === 'function' ? cb : noop;

    return function (list) {
      return new Promise(function (resolve, reject) {
        list = list.slice(0);

        var doNext = function doNext(val) {
          if (list.length) {
            (function () {
              var item = list.shift();

              var fn = typeof item === 'function' ? cb(item) : cb;

              var ret = void 0;
              if ('then' in Object(val)) {
                ret = val.then(function (res) {
                  return fn(res, item);
                });
              } else {
                ret = Promise.resolve(fn(val, item));
              }

              ret.then(doNext, reject);
            })();
          } else {
            resolve(val);
          }
        };

        doNext(initial);
      });
    };
  };
});